"""
Type `Graphique`, qui inclue un graphique avec un point d'ancrage.
"""

from __future__ import annotations

from pytamaro.graphic import Graphic

Graphique = Graphic
"""
Un graphique (image) avec un point à ancrer.

Le point d'ancrage est utilisé pour les opérations suivantes:

- rotation (pour déterminer le centre de rotation)
- composition de graphiques (deux graphiques sont composés en alignant leur point
  d'ancrage).
"""



"""
`Color` type, functions to produce colors, and constants for important colors.
"""

from dataclasses import dataclass
from functools import cached_property

from skia import Color4f

from pytamaro.localization import translate


@dataclass(frozen=True)
class Color:
    """
    Represents a color.
    A color also has a degree of opacity,
    from completely transparent (like the color `transparent`)
    to completely opaque (like the color `red`).
    """
    red: int      # [0-255]
    green: int    # [0-255]
    blue: int     # [0-255]
    alpha: float  # [0-1]

    @cached_property
    def skia_color(self) -> Color4f:
        """
        Returns the current color as a Skia color.

        :meta private:
        :returns: a Skia color
        """
        return Color4f(self.red / 255, self.green / 255, self.blue / 255, self.alpha)

    def __repr__(self) -> str:
        from pytamaro.color_names import \
            _known_colors  # pylint: disable=cyclic-import,import-outside-toplevel
        maybe_known_color = _known_colors.get(self)
        if maybe_known_color:
            return translate(maybe_known_color)
        alpha_repr = "" if self.alpha == 1 else f", {self.alpha}"
        return f"{translate('rgb_color')}({self.red}, {self.green}, {self.blue}{alpha_repr})"



"""
Noms de points particuliers, qui peuvent être utilisé comme des points d'ancrage pour des
graphiques.
"""

from __future__ import annotations

from pytamaro.point_names import (top_left, top_center, top_right,
                                  center_left, center, center_right,
                                  bottom_left, bottom_center, bottom_right)
from pytamaro.fr.point import Point


centre: Point = center
"""Le point centrale du cadre de délimitation du graphique

:meta hide-value:
"""


haut_centre: Point = top_center
"""Le point centrale du côté supérieur du cadre de délimitation du graphique

:meta hide-value:
"""

bas_centre: Point = bottom_center
"""Le point centrale du côté inférieur du cadre de délimitation du graphique

:meta hide-value:
"""

centre_gauche: Point = center_left
"""Le point centrale du côté gauche du cadre de délimitation du graphique

:meta hide-value:
"""


centre_droite: Point = center_right
"""Le point centrale du côté droit du cadre de délimitation du graphique

:meta hide-value:
"""


haut_gauche: Point = top_left
"""Le coin en haut à gauche du cadre de délimitation du graphique

:meta hide-value:
"""

haut_droite: Point = top_right
"""Le coin en haut à droite du cadre de délimitation du graphique

:meta hide-value:
"""

bas_gauche: Point = bottom_left
"""Le coin en bas à gauche du cadre de délimitation du graphique

:meta hide-value:
"""

bas_droite: Point = bottom_right
"""Le coin en bas à droite du cadre de délimitation du graphique

:meta hide-value:
"""



"""
Names of notable points, that can be used as pinning positions for a graphic.
"""

from pytamaro.point import Point, i_hat, j_hat, zero

center: Point = zero
"""The center point of the bounding box

:meta hide-value:
"""


top_center: Point = center.translate(j_hat)
"""The middle point of the top edge of the bounding box

:meta hide-value:
"""

bottom_center: Point = center.translate(j_hat * -1)
"""The middle point of the bottom edge of the bounding box

:meta hide-value:
"""

center_left: Point = center.translate(i_hat * -1)
"""The middle point of the left edge of the bounding box

:meta hide-value:
"""


center_right: Point = center.translate(i_hat)
"""The middle point of the right edge of the bounding box

:meta hide-value:
"""


top_left: Point = center_left.translate(j_hat)
"""The top left corner of the bounding box

:meta hide-value:
"""

top_right: Point = center_right.translate(j_hat)
"""The top right corner of the bounding box

:meta hide-value:
"""


bottom_left: Point = center_left.translate(j_hat * -1)
"""The bottom left corner of the bounding box

:meta hide-value:
"""


bottom_right: Point = center_right.translate(j_hat * -1)
"""The bottom right corner of the bounding box

:meta hide-value:
"""

_known_points = {
    center: "center",
    top_center: "top_center",
    bottom_center: "bottom_center",
    center_left: "center_left",
    center_right: "center_right",
    top_left: "top_left",
    top_right: "top_right",
    bottom_left: "bottom_left",
    bottom_right: "bottom_right",
}
